from __future__ import annotations

import privateprefs.internal.data_serializer as _internal


def load(key: str) -> str | None:
    """
    Loads a value for a given key.
    :param key: A lookup key
    :return: The stored value or None
    """
    return _internal.load(key)


def load_dict(keys: list = None) -> dict:
    """
    Loads multiple values for the given keys.
    :param keys: A list of Keys
    :return: A dict of key-value pairs
    """
    return _internal.load_dict(keys)


def delete_all() -> None:
    """
    Deletes all stored key-value pairs.
    :return: None
    """
    _internal.delete_all()


def delete(key: str) -> None:
    """
    Delete the value for a given key.
    :param key: The key to delete the value of
    :return: None
    """
    _internal.delete(key)



from __future__ import annotations

import ast
import pkg_resources

path_to_test_file = pkg_resources.resource_filename(__name__, 'data/prefs.txt')


def load(key: str) -> str | None:
    """
    Loads a value for a given key.
    :param key: A lookup key
    :return: The stored value or None
    """
    dict_form_text_file = load_dict_from_file()
    if key in dict_form_text_file.keys():
        return dict_form_text_file[key]
    else:
        return None


def load_dict(keys: list = None) -> dict:
    """
    Loads multiple values for the given keys.
    :param keys: A list of Keys
    :return: A dict of key-value pairs
    """
    dict_form_text_file = load_dict_from_file()
    if keys is None:
        return dict_form_text_file
    filtered_dict = {}
    for key in keys:
        if key in dict_form_text_file.keys():
            filtered_dict[key] = dict_form_text_file[key]
    return filtered_dict


def delete_all() -> None:
    """
    Deletes all stored key-value pairs.
    :return: None
    """
    save_empty_file()


def delete(key: str) -> None:
    """
    Delete the value for a given key.
    :param key: The key to delete the value of
    :return: None
    """
    loaded_dict = load_dict_from_file()
    loaded_dict.pop(key)
    is_dict_empty = (loaded_dict == {})
    if is_dict_empty:
        save_empty_file()
    else:
        save_dict_to_file(loaded_dict)


def save_dict_to_file(date: dict) -> None:
    """
    Converts a dict into a string and saves it to a .txt file
    :param date: A dict containing key-value pairs
    :return: None
    """
    str_form_text_file = str(date)
    with open(path_to_test_file, "w") as file:
        file.write(str_form_text_file)


def load_dict_from_file() -> dict:
    """
    Loads a string from a .txt file and converts it to the returned dict
    :return: A dict containing key-value pairs
    """
    with open(path_to_test_file, "r") as file:
        str_form_text_file = file.read()
        if str_form_text_file == "":
            dict_form_text_file = {}
        else:
            dict_form_text_file = ast.literal_eval(str_form_text_file)
        return dict_form_text_file


def save_empty_file() -> None:
    """
    Deletes all stored key-value pairs by writing a blank string to stored .txt file
    :return: None
    """
    with open(path_to_test_file, "w") as file:
        file.write("")


def save(key, value) -> None:
    """
    Saves a value inside the given key.
    :param key: The key used to save the value to
    :param value: The value to save
    :return: None
    """
    save_dict({key: value})


def save_dict(data: dict) -> None:
    """
    Saves a dict of key-value pairs.
    :param data: A dict containing key-value pairs
    :return: None
    """
    dict_form_text_file = load_dict_from_file()
    dict_form_text_file.update(data)
    save_dict_to_file(dict_form_text_file)



import pkg_resources
import re

from xml.dom import minidom

# The relative path from this file.
path_to_coverage_xml = pkg_resources.resource_filename(__name__, '../coverage.xml')
path_to_readme = pkg_resources.resource_filename(__name__, '../README.md')

# When the coverage badge is clicked this is a link you're taking to.
url_link = "https://github.com/DarrenHaba/privateprefs/actions"


def get_coverage_percentage() -> int:
    """
    Gets the total coverage percent from the coverage.xml file generated by pytest.
    :return: The total code coverage percentage.
    """
    coverage_xml_doc = minidom.parse(path_to_coverage_xml)
    elements = coverage_xml_doc.getElementsByTagName('coverage')
    coverage_decimal = elements[0].attributes['line-rate'].value
    coverage_percentage = int(float(coverage_decimal) * 100)
    return coverage_percentage


def get_readme_str() -> str:
    """
    Reads/Gets the README.md file as a text string
    :return: The README.md as a string
    """
    with open(path_to_readme, "r") as file:
        readme_str = file.read()
        return readme_str


def set_readme_str(readme_str: str) -> None:
    """
    Writs/Sets the README.md from a text string
    :return: None
    """
    with open(path_to_readme, "w") as file:
        file.write(readme_str)


def get_badge_link(percent: int,
                   link: str,
                   title: str = "Coverage",
                   color: str = None,
                   symbol: str = "%25",
                   green: str = "31c653",
                   yellow: str = "yellow",
                   red: str = "red") -> str:
    """
    Generates a custom badge using the shields.io API, we manually set the parameters to make
    this a coverage badge, but we could make any kind of badge using different parameters.
    See docs here: https://shields.io/category/coverage
    :param percent: The code coverage percentage
    :param link: The URL opened when badge is clicked
    :param title: The text/title on the left side of the badge
    :param color: The badge color, if set the badge will never change from this color
    :param symbol: The symbol appended to the end of the percentage, defaults to "%25" (for percentage)
    :param green: The badge color when coverage percent is over 85%
    :param yellow: The badge color when coverage percent is between 60% and 85%
    :param red: The badge color when coverage percent is under 60%
    :return: A string that will show up as a badge in README.md file
    """
    if color is None:
        if 85 < percent <= 100:
            color = green
        elif 60 < percent <= 85:
            color = yellow
        else:
            color = red
    return f"[![Pytest - Coverage](https://img.shields.io/badge/{title}-{percent}{symbol}-{color})]({link})"


def generate_badge() -> None:
    """
    Generates a coverage badge, searches and replaces the existing badge in the README.md file
    :return: None
    """
    pct = get_coverage_percentage()
    regex_pattern = r"\[\!\[Pytest - Coverage\].*"
    new_readme_str = re.sub(regex_pattern, get_badge_link(pct, url_link), get_readme_str())
    set_readme_str(new_readme_str)


if __name__ == "__main__":
    # When the CI runs this module, so we need to trigger the badge generator here.
    generate_badge()



import pytest
import privateprefs.internal.data_serializer as ds
import privateprefs as prefs

test_key = "test key"
test_value = "test value"
test_dict = {'test key': 'test value'}
test_dict_as_str = "{'test key': 'test value'}"


@pytest.fixture(autouse=True)
def setup_and_teardown():
    # set up
    prefs.delete_all()
    yield
    # tear down
    prefs.delete_all()


def _load_test_file_str():
    with open(ds.path_to_test_file, "r") as file:
        return file.read()


def _save_dict_str_to_file(dict_str):
    with open(ds.path_to_test_file, "w") as file:
        file.write(dict_str)


def test_load():
    _save_dict_str_to_file(test_dict_as_str)
    assert prefs.load(test_key) == test_value


def test_load_default_value_null():
    _save_dict_str_to_file("")
    assert prefs.load(test_key) is None


def test_load_from_empty_file():
    _save_dict_str_to_file("")
    assert prefs.load(test_key) is None


def test_load_dict_filtered():
    _save_dict_str_to_file("{'key1': 'value1', 'key2': 'value2', 'key3': 'value3'}")
    ret = prefs.load_dict(['key1', 'key3'])
    assert ret['key1'] == 'value1' and ret['key3'] == 'value3'


def test_load_dict_not_filtered():
    _save_dict_str_to_file("{'key1': 'value1', 'key2': 'value2', 'key3': 'value3'}")
    ret = prefs.load_dict(None)
    assert ret['key1'] == 'value1' and ret['key2'] == 'value2' and ret['key3'] == 'value3'


def test_load_dict_with_only_key_in_dict_text_file():
    _save_dict_str_to_file("{'key1': 'value1', 'key2': 'value2', 'key3': 'value3'}")
    ret = prefs.load_dict(['key1', 'key4'])
    assert ret['key1'] == 'value1' and "key4" not in ret.keys()


def test_load_dict_from_empty_file():
    _save_dict_str_to_file("")
    assert prefs.load_dict(test_key) == {}


def test_delete_all():
    ds.save(test_key, test_value)
    prefs.delete_all()
    assert prefs.load(test_key) is None


def test_delete():
    ds.save(test_key, test_value)
    ds.save("other test key", "other test value")
    did_save_value = prefs.load(test_key) == test_value
    prefs.delete(test_key)
    did_delete_value = prefs.load(test_key) is None
    did_not_delete_other_test_value = prefs.load("other test key") == "other test value"
    assert all([did_save_value, did_delete_value, did_not_delete_other_test_value])


def test_delete_empty_dict():
    ds.save(test_key, test_value)
    did_save_value = prefs.load(test_key) == test_value
    prefs.delete(test_key)
    did_delete_value = prefs.load(test_key) is None
    assert all([did_save_value, did_delete_value])



import pytest

from privateprefs.internal.cli import main

test_key = "test key"
test_value = "test value"


@pytest.fixture(autouse=True)
def setup_and_teardown():
    # set up
    main(["delete", "--all"])
    yield
    # tear down
    main(["delete", "--all"])


def test_save(capsys):
    with capsys.disabled():
        main(["save", test_key, test_value])
    main(["load", test_key])
    capture = capsys.readouterr()
    assert capture.out.__contains__(test_value)


def test_load(capsys):
    with capsys.disabled():
        main(["save", test_key, test_value])
    main(["load", test_key])
    capture = capsys.readouterr()
    assert capture.out.__contains__(test_value)


def test_list(capsys):
    with capsys.disabled():
        main(["save", test_key, test_value])
    main(["list"])
    capture = capsys.readouterr()
    contains_test_key = capture.out.__contains__(test_key)
    contains_test_value = capture.out.__contains__(test_value)
    assert all([contains_test_key, contains_test_value])


def test_list_empty(capsys):
    main(["list"])
    capture = capsys.readouterr()
    displays_empty_list = capture.out.__contains__("list is empty: (no key-value pairs saved yet)")
    assert displays_empty_list


def test_delete_all(capsys):
    with capsys.disabled():
        main(["save", test_key, test_value])
        main(["save", "other key", "other value"])
    main(["delete", "--all"])
    capture = capsys.readouterr()
    all_key_value_deleted = capture.out.__contains__("all prefs deleted")
    assert all_key_value_deleted


def test_delete_key(capsys):
    with capsys.disabled():
        main(["save", test_key, test_value])
    main(["delete", test_key])
    capture = capsys.readouterr()
    test_value_deleted = capture.out.__contains__(test_value)
    assert test_value_deleted


def test_delete__no_args_given_error(capsys):
    try:
        main(["delete"])
    except SystemExit:
        assert True
    else:
        assert False



import argparse
# import privateprefs as prefs

# from privateprefs.privateprefs import _save
import privateprefs.internal.data_serializer as ds


def _save_cli(key: str, value: str) -> None:
    """
    Saves a value for a given key.
    :param key: The key used to save the value to
    :param value: The value to save/write to persistent storage
    :return: None
    """
    # noinspection PyProtectedMember
    ds.save(key, value)
    print(f"saved key='{key}' value='{value}'")


def _load_cli(key: str) -> None:
    """
    Loads a value from a given key.
    :param key: The key to load a value from
    :return: None
    """
    value = ds.load(key)
    print(f"loaded key='{key}' value='{value}'")


def _delete_cli(key: str, delete_all: bool = False) -> None:
    """
    Deletes a key-value pair, or deletes all stored key-value pairs.
    :param key: The key to be deleted
    :param delete_all: Flag is set to true will delete all key-value pairs
    :return: None
    """
    if delete_all:
        ds.delete_all()
        print(f"all prefs deleted")
    else:
        print(f"deleted key='{key}' value='{ds.load(key)}'")
        ds.delete(key)


def _list_cli() -> None:
    """
    Displays a list of all saved key-value pairs.
    :return: None
    """
    print_list()


def print_list() -> None:
    """
    Prints out a list of all saved key-value pairs.
    :return: None
    """
    print()
    print("stored (key  :  value)")
    print("-------------------------------------------------------------")
    d = ds.load_dict()
    if len(d) > 0:
        for key, value in ds.load_dict().items():
            print(key, '  :  ', value)
    else:
        print("list is empty: (no key-value pairs saved yet)")
    print("-------------------------------------------------------------")
    print()


def main(argv=None) -> None:
    """
    This main function is the entry point for our CLI in our package.

    In the project.toml file under [project.scripts] is a line code that invokes
    this method. This method instantiate argparse and instantiates sub-commands
    then injects the sub-commands into the global namespace cache
    :param argv: Injected arguments when running unit tests, and None when called
    from the command line
    :return: None
    """

    # Instantiate argparse and a sub-parsers
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(dest='subparser')

    # To make argparse sub-parsers easier to deal with, we set up one function per subparsers.

    # The Save sub-parsers.
    # A function called 'save_' will be dynamically called when the 'save' command is invoked
    parser_save = subparsers.add_parser("save")
    parser_save.add_argument("key")
    parser_save.add_argument("value")

    # The Load sub-parsers.
    # A function called 'load_' will be dynamically called when the 'load' command is invoked
    parser_load = subparsers.add_parser("load")
    parser_load.add_argument("key")

    # The List sub-parsers.
    # A function called 'list_' will be dynamically called when the 'list' command is invoked
    subparsers.add_parser("list")

    # The Delete sub-parsers.
    # A function called 'delete_' will be dynamically called when the 'delete' command is invoked
    # Note that only the 'key' or '--all' argument is required, depending on if we want to delete
    # single key-value pair or all key-value pairs.
    parser_delete = subparsers.add_parser("delete")
    parser_delete.add_argument('key', nargs='?')
    parser_delete.add_argument("--all",
                               dest="delete_all",
                               action="store_true")
    args = parser.parse_args(argv)
    no_args_given = args.subparser == "delete" and args.key is None and args.delete_all is False
    if no_args_given:
        raise parser.error("you must enter a key string argument or enter the '--all' flag")

    # Extract a dict containing the name of the sub processor invoked and its arguments
    kwargs = vars(parser.parse_args(argv))

    # Extract the name of the subcommand being invoked, note we pop/remove the subcommand name,
    # so now kwargs will be just the given arguments without the subcommand in the dict.
    func_name_to_call = kwargs.pop('subparser')  # will be: save, load, delete, etc.

    # The private functions end with an underscore and in with _cli, so we add it here.
    func_name_to_call = f"_{func_name_to_call}_cli"

    # We need to dynamically call one of the save(), load(), delete(), etc functions,
    # We dynamically call the function from the globals namespace dictionary, passing in the arguments.
    globals()[func_name_to_call](**kwargs)


if __name__ == '__main__':
    main()
